@@ -16,7 +16,7 @@ import glob
 import os
 import csv
 import pandas as pd
-from ..utils import common_functions as c_f
+from ..utils import common_functions as c_f, constants as const
 from .base_runner import BaseRunner
 from .single_experiment_runner import SingleExperimentRunner
 import pytorch_metric_learning.utils.logging_presets as logging_presets
@@ -27,11 +27,8 @@ import collections
 import json
 logging.info("Done importing packages in bayes_opt_runner")
 
-BAYESIAN_KEYWORDS=("~BAYESIAN~", "~LOG_BAYESIAN~", "~INT_BAYESIAN~")
-RESUME_FAILURE="RESUME_FAILURE"
 
-
-def set_optimizable_params_and_bounds(args_dict, bayes_params, parent_key, keywords=BAYESIAN_KEYWORDS):
+def set_optimizable_params_and_bounds(args_dict, bayes_params, parent_key, keywords=const.BAYESIAN_KEYWORDS):
     for k, v in args_dict.items():
         if not isinstance(v, dict):
             for keyword in keywords:
@@ -69,12 +66,6 @@ def open_log(log_paths):
     return ax_client
 
 
-def remove_keywords(YR):
-    emag_utils.remove_key_word_recursively(YR.args.__dict__, "~OVERRIDE~")
-    for keyword in BAYESIAN_KEYWORDS:
-        emag_utils.remove_key_word_recursively(YR.args.__dict__, keyword)
-
-
 class BayesOptRunner(BaseRunner):
     def __init__(self, bayes_opt_iters, reproductions, **kwargs):
         super().__init__(**kwargs)
@@ -90,8 +81,6 @@ class BayesOptRunner(BaseRunner):
         self.ax_log_folder = os.path.join(self.bayes_opt_root_experiment_folder, "bayes_opt_ax_logs")
         self.best_parameters_filename = os.path.join(self.bayes_opt_root_experiment_folder, "best_parameters.yaml")
         self.most_recent_parameters_filename = os.path.join(self.bayes_opt_root_experiment_folder, "most_recent_parameters.yaml")
-        self.accuracy_report_detailed_filename = os.path.join(self.bayes_opt_root_experiment_folder, "accuracy_report_detailed.yaml")
-        self.accuracy_report_filename = os.path.join(self.bayes_opt_root_experiment_folder, "accuracy_report.yaml")
         self.bayes_opt_table_name = "bayes_opt"
         self.set_YR(use_super=False)
 
@@ -189,7 +178,8 @@ class BayesOptRunner(BaseRunner):
         return best_sub_experiment_name
 
     def create_accuracy_report(self, best_sub_experiment_name):
-        dummy_api_parser = self.get_api_parser(self.YR.args)
+        dummy_YR = self.read_yaml_and_find_bayes(find_bayes_params=False, merge_argparse=True)
+        dummy_api_parser = self.get_api_parser(dummy_YR.args)
         eval_record_group_dicts = dummy_api_parser.get_eval_record_name_dict(return_all=True)
         global_record_keeper, _, _ = logging_presets.get_record_keeper(self.csv_folder, self.tensorboard_folder, self.global_db_path, "", False)
         exp_names = glob.glob(os.path.join(self.bayes_opt_root_experiment_folder, "%s*"%best_sub_experiment_name))
@@ -197,7 +187,7 @@ class BayesOptRunner(BaseRunner):
         exp_names = [os.path.basename(e) for e in exp_names]
         results, summary = {}, {}
 
-        for eval_type in c_f.if_str_convert_to_singleton_list(self.YR.args.meta_testing_method):
+        for eval_type in c_f.if_str_convert_to_singleton_list(dummy_YR.args.meta_testing_method):
             results[eval_type] = {}
             summary[eval_type] = collections.defaultdict(lambda: collections.defaultdict(list))
             table_name = eval_record_group_dicts[eval_type]["test"]
@@ -205,36 +195,39 @@ class BayesOptRunner(BaseRunner):
             for exp in exp_names:
                 results[eval_type][exp] = {}
                 exp_id = global_record_keeper.record_writer.global_db.get_experiment_id(exp)
-                base_query = "SELECT * FROM %s WHERE experiment_id=? AND id=? AND is_trained=?"%table_name
-                max_id_query = "SELECT max(id) FROM %s WHERE experiment_id=? AND is_trained=?"%table_name
+                base_query = "SELECT * FROM {} WHERE experiment_id=? AND id=? AND {}=?".format(table_name, const.TRAINED_STATUS_COL_NAME)
+                max_id_query = "SELECT max(id) FROM {} WHERE experiment_id=? AND {}=?".format(table_name, const.TRAINED_STATUS_COL_NAME)
                 qs = {}
 
-                for key, is_trained in [("trained", 1), ("untrained", 0)]:
-                    max_id = global_record_keeper.query(max_id_query, values=(exp_id, is_trained), use_global_db=True)[0]["max(id)"]
-                    q = global_record_keeper.query(base_query, values=(exp_id, max_id, is_trained), use_global_db=True)
+                for trained_status in [const.UNTRAINED_TRUNK, const.UNTRAINED_TRUNK_AND_EMBEDDER, const.TRAINED]:
+                    max_id = global_record_keeper.query(max_id_query, values=(exp_id, trained_status), use_global_db=True)[0]["max(id)"]
+                    q = global_record_keeper.query(base_query, values=(exp_id, max_id, trained_status), use_global_db=True)
                     if len(q) > 0:
-                        qs[key] = q[0]
+                        qs[trained_status] = q[0]
 
-                for is_trained, v1 in qs.items():
+                for trained_status, v1 in qs.items():
                     q_as_dict = dict(v1)
-                    results[eval_type][exp][is_trained] = q_as_dict
+                    results[eval_type][exp][trained_status] = q_as_dict
                     for acc_key, v2 in q_as_dict.items():
-                        if all(not acc_key.startswith(x) for x in ["epoch", "is_trained", "SEM", "id", "experiment_id", "timestamp"]):
-                            summary[eval_type][is_trained][acc_key].append(v2)
+                        if all(not acc_key.startswith(x) for x in [const.TRAINED_STATUS_COL_NAME, "epoch", "SEM", "id", "experiment_id", "timestamp"]):
+                            summary[eval_type][trained_status][acc_key].append(v2)
 
 
-            for is_trained, v1 in summary[eval_type].items():
+            for trained_status, v1 in summary[eval_type].items():
                 for acc_key in v1.keys():
                     v2 = v1[acc_key]
                     mean = np.mean(v2)
                     cf_low, cf_high = scipy_stats.t.interval(0.95, len(v2)-1, loc=np.mean(v2), scale=scipy_stats.sem(v2)) #https://stackoverflow.com/a/34474255
                     cf_width = mean-cf_low
-                    summary[eval_type][is_trained][acc_key] = {"mean": float(mean), 
+                    summary[eval_type][trained_status][acc_key] = {"mean": float(mean), 
                                                                     "95%_confidence_interval": (float(cf_low), float(cf_high)),
                                                                     "95%_confidence_interval_width": float(cf_width)}
 
-        c_f.write_yaml(self.accuracy_report_detailed_filename, results, open_as="w")
-        c_f.write_yaml(self.accuracy_report_filename, json.loads(json.dumps(summary)), open_as="w")
+        eval_name = c_f.first_val_of_dict(dummy_api_parser.get_eval_record_name_dict(eval_type=const.NON_META, return_base_record_group_name=True))
+        detailed_report_filename = os.path.join(self.bayes_opt_root_experiment_folder, "detailed_report_{}.yaml".format(eval_name))
+        report_filename = os.path.join(self.bayes_opt_root_experiment_folder, "report_{}.yaml".format(eval_name))
+        c_f.write_yaml(detailed_report_filename, results, open_as="w")
+        c_f.write_yaml(report_filename, json.loads(json.dumps(summary)), open_as="w")
 
 
     def update_bayes_opt_search_space(self, ax_client):
@@ -260,20 +253,23 @@ class BayesOptRunner(BaseRunner):
         model = Models.GPEI(experiment=ax_client.experiment, data=ax_client.experiment.fetch_data())
         html_elements = [plot_config_to_html(ax_client.get_optimization_trace())]
         model_params = get_range_parameters(model)
-        if len(model_params) > 1:
-            html_elements.append(plot_config_to_html(interact_contour(model=model, metric_name=self.YR.args.eval_primary_metric)))
-        else:
-            html_elements.append(plot_config_to_html(interact_slice(model=model, param_name=model_params[0].name, metric_name=self.YR.args.eval_primary_metric)))
+        try:
+            if len(model_params) > 1:
+                html_elements.append(plot_config_to_html(interact_contour(model=model, metric_name=self.YR.args.eval_primary_metric)))
+            else:
+                html_elements.append(plot_config_to_html(interact_slice(model=model, param_name=model_params[0].name, metric_name=self.YR.args.eval_primary_metric)))
+        except TypeError:
+            pass
         with open(os.path.join(self.bayes_opt_root_experiment_folder, "optimization_plots.html"), 'w') as f:
             f.write(render_report_elements(self.experiment_name, html_elements))
 
 
-    def read_yaml_and_find_bayes(self, find_bayes_params=True):
+    def read_yaml_and_find_bayes(self, find_bayes_params=True, merge_argparse=False):
         YR = self.setup_yaml_reader()
         bayes_opt_config_exists = os.path.isdir(YR.args.place_to_save_configs)
 
         config_paths = self.get_saved_config_paths(YR.args) if bayes_opt_config_exists else self.get_root_config_paths(YR.args)
-        merge_argparse = self.merge_argparse_when_resuming if bayes_opt_config_exists else True
+        merge_argparse = (self.merge_argparse_when_resuming or merge_argparse) if bayes_opt_config_exists else True
         YR.args, _, YR.args.dict_of_yamls = YR.load_yamls(config_paths = config_paths, 
                                                         max_merge_depth = float('inf'), 
                                                         merge_argparse = merge_argparse)
@@ -323,7 +319,7 @@ class BayesOptRunner(BaseRunner):
             logging.error(repr(e))
             logging.warning("Could not resume training for %s"%YR.args.experiment_name)
             self.delete_sub_experiment_folder(YR.args.experiment_name)
-            output = RESUME_FAILURE
+            output = const.RESUME_FAILURE
         return output
 
 
@@ -341,8 +337,8 @@ class BayesOptRunner(BaseRunner):
             self.delete_sub_experiment_folder(sub_experiment_name)
             parameter_load_successful = False
 
-        output = self.try_resuming(local_YR) if parameter_load_successful else RESUME_FAILURE
-        return self.run_new_experiment(parameters, sub_experiment_name) if output == RESUME_FAILURE else output
+        output = self.try_resuming(local_YR) if parameter_load_successful else const.RESUME_FAILURE
+        return self.run_new_experiment(parameters, sub_experiment_name) if output == const.RESUME_FAILURE else output
 
 
     def run_new_experiment(self, parameters, sub_experiment_name):
@@ -378,11 +374,11 @@ class BayesOptRunner(BaseRunner):
             local_YR = self.get_simplified_yaml_reader("%s_reproduction%d"%(sub_experiment_name, i))
             local_YR.args.reproduce_results = self.get_sub_experiment_path(sub_experiment_name)
             local_YR.args.resume_training = None
-            output = RESUME_FAILURE
+            output = const.RESUME_FAILURE
             if os.path.isdir(local_YR.args.experiment_folder):
                 local_YR.args.resume_training = self.get_resume_training_value()
                 output = self.try_resuming(local_YR, reproduction=True)
-            if output == RESUME_FAILURE:
+            if output == const.RESUME_FAILURE:
                 SER = self.get_single_experiment_runner()
                 starting_fresh_hook = self.starting_fresh(local_YR.args.experiment_name)
                 SER.reproduce_results(local_YR, starting_fresh_hook=starting_fresh_hook)
@@ -394,9 +390,12 @@ class BayesOptRunner(BaseRunner):
 
     
     def get_single_experiment_runner(self):
-        return SingleExperimentRunner(root_experiment_folder=self.bayes_opt_root_experiment_folder, 
+        SER = SingleExperimentRunner(root_experiment_folder=self.bayes_opt_root_experiment_folder, 
                                     root_config_folder=self.YR.args.place_to_save_configs, 
                                     dataset_root=self.dataset_root,
                                     pytorch_home=self.pytorch_home, 
                                     global_db_path=self.global_db_path,
-                                    merge_argparse_when_resuming=self.merge_argparse_when_resuming)
\ No newline at end of file
+                                    merge_argparse_when_resuming=self.merge_argparse_when_resuming)
+        
+        SER.pytorch_getter = self.pytorch_getter
+        return SER
\ No newline at end of file
