@@ -27,6 +27,7 @@ class SingleExperimentRunner(BaseRunner):
         return run_output
 
     def run_new_experiment_or_resume(self, YR):
+        # merge_argparse at the beginning of training, or when evaluating
         merge_argparse = self.merge_argparse_when_resuming if YR.args.resume_training else True
         args, _, args.dict_of_yamls = YR.load_yamls(self.determine_where_to_get_yamls(YR.args), 
                                                     max_merge_depth=float('inf'), 
@@ -44,11 +45,8 @@ class SingleExperimentRunner(BaseRunner):
                                                     merge_argparse=self.merge_argparse_when_resuming)
 
         # check if there were config diffs if training was resumed
-        if YR.args.special_split_scheme_name:
-            base_split_name = YR.args.special_split_scheme_name
-        else:
-            base_split_name = d_u.get_base_split_name(YR.args.test_size, YR.args.test_start_idx, YR.args.num_training_partitions)
-        resume_training_dict = c_f.get_all_resume_training_config_diffs(configs_folder, base_split_name)
+        temp_split_manager = self.pytorch_getter.get("split_manager", yaml_dict=args.split_manager)
+        resume_training_dict = c_f.get_all_resume_training_config_diffs(configs_folder, temp_split_manager)
 
         if len(resume_training_dict) > 0:
             for sub_folder, num_epochs_dict in resume_training_dict.items():
